--
-- LSUpgradeShop
--
-- @author  ANDR55
-- Version 0.1
-- @date 12.10.2014

local directory = g_currentModDirectory;
LSUpgradeShop = {};


function LSUpgradeShop:loadMap(name)

	self.initialize = false;
	
	self.overlayDesktop  = Overlay:new("PDA",            "dataS2/missions/hud_pda_frame.png", -0.03,  0.096, 0.75, 0.75);
	self.overlayDesktop2 = Overlay:new("PDABackground",  "dataS2/missions/hud_pda_bg.png",     0.042, 0.174, 0.6,  0.6);
	
	self.showGui = false;
	
end;

function LSUpgradeShop:update(dt)

	if not self.initialize then
		self.initialize = true;
	end;
	
	if InputBinding.hasEvent(InputBinding.LSUpgrade_SHOW) then
		LSUpgradeShop:ShowingGui(true);
	end;
	
	if InputBinding.hasEvent(InputBinding.LSUpgrade_SHOW2) then
		LSUpgradeShop:ShowingGui(false);
	end;
	
	if self.showGui then
	
		for _, upgrade in pairs (self.activeVehicles[self.ActiveVehicle].upgrades) do
	
			if upgrade.refreshDisplay then
				self:RenderUpgrades();
				upgrade.refreshDisplay = false;
				break;
			end;
		end;
	end;
	
end;

function LSUpgradeShop:ShowingGui(show)

	if g_currentMission.controlledVehicle ~= nil then

		self.showGui = show;
		
		if show then
			
			self.activeVehicles = {}
			self.activeStoreItems = {}
			
			LSUpgradeShop:AddVehicle(g_currentMission.controlledVehicle);
			
			self.overlayDesktopVehicle = {}
			
			self.menuButtons 		= {}
			self.menuUpgradeButtons = {}
			self.menuUpgradeText    = {}
			
			for i=1, #self.activeVehicles do
				
				self:addButton("",   self.activeStoreItems[i].imageActive,   self.SetActiveVehicle, i, (0.0665 * i) + 0.0035, 0.685, 0.06, 0.06);
			end;
			
			LSUpgradeShop:SetActiveVehicle(1);
			
			self:addButton("",   "dataS2/menu/button_left_normal.png",  self.SwitchActiveVehicle, -1, 0.05, 0.672, 0.017, 0.08);
			self:addButton("",   "dataS2/menu/button_right_normal.png", self.SwitchActiveVehicle,  1, 0.6,  0.672, 0.017, 0.08);
			
			
			self:addButton("",   "dataS2/menu/button_up_normal.png",   self.SwitchOffsetUpgrade, -1, 0.23, 0.58, 0.40, 0.03);
			self:addButton("",   "dataS2/menu/button_down_normal.png", self.SwitchOffsetUpgrade,  1, 0.23, 0.2,  0.40, 0.03);
			
			self:addButton("",   "dataS2/menu/slot_selected.png",   nil, nil, 0.245, 0.47, 0.37, 0.11);
			self:addButton("",   "dataS2/menu/slot_selected.png",   nil, nil, 0.245, 0.35, 0.37, 0.11);
			self:addButton("",   "dataS2/menu/slot_selected.png",   nil, nil, 0.245, 0.23, 0.37, 0.11);
			
			self:addButton(g_i18n:getText("LSUpgrade_leave"),   "dataS2/menu/button_normal.png",  self.ShowingGui, false, 0.05, 0.23, 0.11, 0.06);
		end;
		
		
		if show then
			g_gui.currentGui = self;
		else
			g_gui.currentGui = nil;
		end;
		
		g_mouseControlsHelp.active = not show;
		InputBinding.setShowMouseCursor(show);
	end;
	
end;

function LSUpgradeShop:AddVehicle(vehicle)

	local storeItem = nil;
	
	for key,value in pairs(g_gui.guis.ShopScreen.target.storeItems) do
	
		if vehicle.configFileName == value.xmlFilename then
		
			storeItem = value;
			break;
		end;
	end;
	
	if storeItem ~= nil then
		self.activeVehicles[#self.activeVehicles + 1] = vehicle
		self.activeStoreItems[#self.activeStoreItems + 1] = storeItem
	end;

	
	for i=table.getn(vehicle.attachedImplements), 1, -1 do
		LSUpgradeShop:AddVehicle(vehicle.attachedImplements[i].object);
	end;
	
end;

function LSUpgradeShop:SwitchActiveVehicle(i)

	LSUpgradeShop:SetActiveVehicle(self.ActiveVehicle + i);
end;

function LSUpgradeShop:SetActiveVehicle(i)

	if i >= 1                    and
	   i <= #self.activeVehicles then
	
		self.ActiveVehicle = i;
		
		--self.overlayDesktopActiveVehicleFrame = Overlay:new("ActiveVehicleFrame", "dataS2/missions/hud_pda_frame.png" , (0.05 * i) - 0.01, 0.6, 0.01, 0.014);

		self.overlayDesktopActiveVehicle = Overlay:new("ActiveVehicle", self.activeStoreItems[i].imageActive , 0.05, 0.43, 0.18, 0.18);
		
		self:SetOffsetUpgrade(1);
	end;
		
end;

function LSUpgradeShop:RenderUpgrades()

	self.menuUpgradeButtons = {};
	self.menuUpgradeText = {};
	
	local position = 0;
	
	if self.offsetUpgrade ~= nil then
	for x = self.offsetUpgrade, self.offsetUpgrade + 2 do
	
		local upgrade = self.activeVehicles[self.ActiveVehicle].upgrades[x];
	
		if upgrade == nil then
			break;
		end;
	
		self:addUpgradeText(g_i18n:getText("LSUpgrade_"..upgrade.name).." "..g_i18n:getText("LSUpgrade_level").." "..tostring(upgrade.level), true, 0.255, 0.55 - (position * 0.12), 0.025);
		
		self:addUpgradeText(g_i18n:getText("LSUpgrade_start_value") , false, 0.255, 0.52 - (position * 0.12), 0.020);
		self:addUpgradeText(g_i18n:getText("LSUpgrade_actual_value"), false, 0.255, 0.50 - (position * 0.12), 0.020);
		
		if Utils.getNoNil(tonumber(upgrade.level), 0) < tonumber(upgrade.maxlevel) then
			self:addUpgradeText(g_i18n:getText("LSUpgrade_after_upgrade"), false, 0.255, 0.48 - (position * 0.12), 0.020);
		end;
		
		self:addUpgradeText(tostring(math.floor(upgrade.value_orig * Utils.getNoNil(upgrade.displaymultiplier, 1)))                                                                                               , false, 0.41, 0.52 - (position * 0.12), 0.020, RenderText.ALIGN_RIGHT);
		self:addUpgradeText(tostring(math.floor((upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) *  Utils.getNoNil(upgrade.level, 0))))      * Utils.getNoNil(upgrade.displaymultiplier, 1))), false, 0.41, 0.50 - (position * 0.12), 0.020, RenderText.ALIGN_RIGHT);
		
		if Utils.getNoNil(tonumber(upgrade.level), 0) < tonumber(upgrade.maxlevel) then
			self:addUpgradeText(tostring(math.floor((upgrade.value_orig + (upgrade.value_orig * ((upgrade.increase / 100) * (Utils.getNoNil(upgrade.level, 0) + 1)))) * Utils.getNoNil(upgrade.displaymultiplier, 1))), false, 0.41, 0.48 - (position * 0.12), 0.020, RenderText.ALIGN_RIGHT);
		end;
		
		self:addUpgradeText(Utils.getNoNil(upgrade.unit , ""), false, 0.415, 0.52 - (position * 0.12), 0.020);
		self:addUpgradeText(Utils.getNoNil(upgrade.unit , ""), false, 0.415, 0.50 - (position * 0.12), 0.020);
		
		if Utils.getNoNil(tonumber(upgrade.level), 0) < tonumber(upgrade.maxlevel) then
			self:addUpgradeText(Utils.getNoNil(upgrade.unit , ""), false, 0.415, 0.48 - (position * 0.12), 0.020);
		end;
		
		local price = self.activeStoreItems[self.ActiveVehicle].price * ((upgrade.costpercent + (upgrade.costpercentincrease * upgrade.level)) / 100);
		
		if Utils.getNoNil(tonumber(upgrade.level), 0) < tonumber(upgrade.maxlevel) then
			self:addUpgradeText(g_i18n:formatMoney(price), true, 0.61, 0.55  - (position * 0.12), 0.025, RenderText.ALIGN_RIGHT);
		end;

		local enabled = (tonumber(Utils.getNoNil(upgrade.level, 0)) < upgrade.maxlevel) and (g_currentMission.missionStats.money >= price);
		self:addUpgradeButton("Upgrade",   "dataS2/menu/button_normal.png",   LSUpgradeShop.UpUpgrade, upgrade, 0.53, 0.47 - (position * 0.12), 0.086, 0.06, enabled);
	
		position = position + 1;
	end;
	end;

end;

function LSUpgradeShop:SetOffsetUpgrade(i)

	if i >= 1                                                 and
	   i <= #self.activeVehicles[self.ActiveVehicle].upgrades then

		if #self.activeVehicles[self.ActiveVehicle].upgrades  <= 3 then
		
			self.offsetUpgrade = 1;
		
		end;
	   
		if #self.activeVehicles[self.ActiveVehicle].upgrades > 3      and
		   i <= #self.activeVehicles[self.ActiveVehicle].upgrades - 2 then
	   
			self.offsetUpgrade = i;
		end;
	end;

	self:RenderUpgrades();
	
end;

function LSUpgradeShop:SwitchOffsetUpgrade(i)

	self:SetOffsetUpgrade(self.offsetUpgrade + i);

end;

function LSUpgradeShop:draw()

	if self.showGui == true then
	
		self.overlayDesktop :render();
		self.overlayDesktop2:render();
	

		--self.overlayDesktopActiveVehicleFrame:render();
		self.overlayDesktopActiveVehicle:render();
		
		if self.ActiveVehicle ~= nil then
		
			setTextAlignment(RenderText.ALIGN_LEFT);
			setTextColor(1, 1, 1, 1);
			setTextBold(true);
			renderText(0.05, 0.63, 0.03, self.activeStoreItems[self.ActiveVehicle].name);
			
			self:renderButtons();
		end;
	end;

end;

function LSUpgradeShop:addButton(buttonText, imgFilename, callback, callbackData, x, y, width, height, mouseButtons)

    if (mouseButtons == nil) then
		mouseButtons = self.MouseButton_Left;
	end;
	
    local overlay = nil;
	
    if (imgFilename ~= nil) then
        overlay = Overlay:new(buttonText, imgFilename, x, y, width, height);
    end;
	
    local button = {visible=(overlay ~= nil), enabled=true, buttonText=buttonText, overlay=overlay, rect={x, y, x + width, y + height}, onClick=callback, callbackData=callbackData, mouseButtons=mouseButtons};
    
	table.insert(self.menuButtons, button);
    return button;
end;

function LSUpgradeShop:addUpgradeButton(buttonText, imgFilename, callback, callbackData, x, y, width, height, enabled, mouseButtons)

    if (mouseButtons == nil) then
		mouseButtons = self.MouseButton_Left;
	end;
	
    local overlay = nil;
	
    if (imgFilename ~= nil) then
        overlay = Overlay:new(buttonText, imgFilename, x, y, width, height);
    end;
	
    local button = {visible=(overlay ~= nil), enabled=enabled, buttonText=buttonText, overlay=overlay, rect={x, y, x + width, y + height}, onClick=callback, callbackData=callbackData, mouseButtons=mouseButtons};
    
	table.insert(self.menuUpgradeButtons, button);
    return button;
end;

function LSUpgradeShop:addUpgradeText(text, bold, x, y, size, align)

    local mtext = {text=text, bold=bold, rect={x, y}, size=size, align=align};
    
	table.insert(self.menuUpgradeText, mtext);
    return mtext;
end;

function LSUpgradeShop:UpUpgrade(upgrade)

	local price = self.activeStoreItems[self.ActiveVehicle].price * ((upgrade.costpercent + (upgrade.costpercentincrease * upgrade.level)) / 100);
	
	
	local user = LSUpgradeShop:GetControllerName(self.activeVehicles[self.ActiveVehicle]);
	
	local userid = 0;
	
	for i = 1, table.getn(g_currentMission.users) do
	
		if g_currentMission.users[i].nickname == user then
			userid = g_currentMission.users[i].userId;
		end
	end;

	
	g_client:getServerConnection():sendEvent(AddMoneyToUserEvent:new(-price, userid, "other"));
	
	g_client:getServerConnection():sendEvent(LSUpgradeEvent:new(self.activeVehicles[self.ActiveVehicle], upgrade.name, upgrade.level + 1))
	
end;

function LSUpgradeShop:GetControllerName(veh)

	local ok = false;
	local user = "";
	

	while ok == false and veh ~= nil do
		if veh.controllerName ~= nil then
			ok = true;
			break;
		end;
		veh = veh.attacherVehicle;
	end;
	
	if veh ~= nil then
		if veh.controllerName ~= nil then
			user = veh.controllerName;
		end;
	end;
	
	return user;

end;

function LSUpgradeShop:renderButtons()
    
    setTextBold(true);
	
    for _,button in pairs(self.menuButtons) do
		
		setTextAlignment(RenderText.ALIGN_LEFT);
		
        if (button.visible) then
            if (button.overlay ~= nil) then
                button.overlay:render();
            end;
            if (button.buttonText ~= nil) then
                if (button.enabled) then
					setTextColor(0.0, 0.0, 0.0, 0.75)
                else
                    setTextColor(0.65, 0.65, 0.65, 0.5);  
                end;
				
                renderText(button.rect[1] + 0.006, button.rect[2] + 0.015, 0.025, button.buttonText);
            end;
        end;
    end;

    for _,button in pairs(self.menuUpgradeButtons) do
	
		setTextAlignment(RenderText.ALIGN_LEFT);
	
        if (button.visible) then
            if (button.overlay ~= nil) then
                button.overlay:render();
            end;
            if (button.buttonText ~= nil) then
                if (button.enabled) then
					setTextColor(0.0, 0.0, 0.0, 0.75)
                else
                    setTextColor(0.65, 0.65, 0.65, 0.5);  
                end;
				
                renderText(button.rect[1] + 0.006, button.rect[2] + 0.015, 0.025, button.buttonText);
            end;
        end;
    end;
	
    for _,mtext in pairs(self.menuUpgradeText) do
	
		if mtext.align ~= nil then
			setTextAlignment(mtext.align);
		else
			setTextAlignment(RenderText.ALIGN_LEFT);
		end;
		
		setTextColor(1, 1, 1, 1);
		setTextBold(mtext.bold);
			
		renderText(mtext.rect[1], mtext.rect[2], mtext.size, mtext.text);
    end;
	
	setTextBold(false);
end;

function LSUpgradeShop:mouseEvent(posX, posY, isDown, isUp, button)

    if self.showGui then
		if isDown then
			if button == 1 then
				for key, button in pairs(self.menuButtons) do
					if button.enabled then
						if button.rect[1] <= posX and posX <= button.rect[3] and button.rect[2] <= posY and posY <= button.rect[4] then
							if button.onClick ~= nil then
								button.onClick(self, button.callbackData);
								break;
							end;
						end;
					end;
				end;
				
				for key, button in pairs(self.menuUpgradeButtons) do
					if button.enabled then
						if button.rect[1] <= posX and posX <= button.rect[3] and button.rect[2] <= posY and posY <= button.rect[4] then
							if button.onClick ~= nil then
								button.onClick(self, button.callbackData);
								break;
							end;
						end;
					end;
				end;
			end;
		end;
	end;
end;

function LSUpgradeShop:deleteMap()
end;

function LSUpgradeShop:delete()
end;

function LSUpgradeShop:keyEvent(unicode, sym, modifier, isDown)
end;

addModEventListener(LSUpgradeShop);